@@ -15,8 +15,8 @@
 #' @format \code{\link{R6Class}} object.
 #' @field p Parameter for correlation
 #' @field p_est Should p be estimated?
-#' @field p_lower Lower bound of logp
-#' @field p_upper Upper bound of logp
+#' @field p_lower Lower bound of p
+#' @field p_upper Upper bound of p
 #' @field p_length length of p
 #' @field s2 variance
 #' @field s2_est Is s2 estimated?
@@ -26,6 +26,11 @@
 #' @field xindex Index of the factor (which column of X)
 #' @field nlevels Number of levels for the factor
 #' @field latentdim Dimension of embedding space
+#' @field pf_to_p_log Logical vector used to convert pf to p
+#' @field p_to_pf_inds Vector of indexes used to convert p to pf
+#' @field offdiagequal What should offdiagonal values be set to when the
+#' indices are the same? Use to avoid decomposition errors, similar to
+#' adding a nugget.
 #' @examples
 #' # Create a new kernel for a single factor with 5 levels,
 #' #  mapped into two latent dimensions.
@@ -91,7 +96,6 @@ LatentFactorKernel <- R6::R6Class(
   public = list(
     p = NULL, # vector
     p_est = NULL,
-    # logp = NULL,
     p_lower = NULL,
     p_upper = NULL,
     p_length = NULL,
@@ -103,13 +107,11 @@ LatentFactorKernel <- R6::R6Class(
     nlevels = NULL,
     latentdim = NULL,
     xindex = NULL,
-    # alpha = NULL,
-    # logalpha = NULL,
-    # logalpha_lower = NULL,
-    # logalpha_upper = NULL,
-    # alpha_est = NULL,
+    pf_to_p_log = NULL,
+    p_to_pf_inds = NULL,
+    offdiagequal = NULL,
     #' @description Initialize kernel object
-    #' @param p Periodic parameter
+    #' @param p Vector of latent variables
     #' @param s2 Initial variance
     #' @param D Number of input dimensions of data
     #' @param p_lower Lower bound for p
@@ -121,10 +123,15 @@ LatentFactorKernel <- R6::R6Class(
     #' @param xindex Index of X to use the kernel on
     #' @param nlevels Number of levels for the factor
     #' @param latentdim Dimension of embedding space
+    #' @param useC Should C code used? Much faster.
+    #' @param offdiagequal What should offdiagonal values be set to when the
+    #' indices are the same? Use to avoid decomposition errors, similar to
+    #' adding a nugget.
     initialize = function(s2=1, D, nlevels, xindex,
                           latentdim,
                           p_lower=0, p_upper=1, p_est=TRUE,
-                          s2_lower=1e-8, s2_upper=1e8, s2_est=TRUE
+                          s2_lower=1e-8, s2_upper=1e8, s2_est=TRUE,
+                          useC=TRUE, offdiagequal=1-1e-6
     ) {
       # Must give in D
       if (missing(D)) {stop("Must give Index kernel D")}
@@ -137,37 +144,38 @@ LatentFactorKernel <- R6::R6Class(
       stopifnot(length(D) == 1, length(nlevels) == 1,
                 length(xindex) == 1, length(latentdim) == 1,
                 D>=1L, nlevels>=2L, xindex>=1L, latentdim>=1)
-      stopifnot(latentdim <= nlevels)
+      # Following avoids redundancies
+      stopifnot(latentdim < nlevels)
 
       self$D <- D
       self$nlevels <- nlevels
       self$xindex <- xindex
       self$latentdim <- latentdim
 
-      # p <- rep(0, D * (D-1) / 2)
-      # p <- rep(1, nlevels - 1)
+      p_to_pf <- c()
+      for (i in 1:nlevels) {
+        n0 <- max(0, latentdim+1-i)
+        nnon0 <- latentdim - n0
+        p_to_pf <- c(p_to_pf, rep(T, nnon0), rep(F, n0))
+      }
+      pf_to_p <- which(p_to_pf)
+      # p_to_pf
+      # pf_to_p
+      # (1:(nlev*nld))[pf_to_p]
+      # Names are backwards, or just unclear
+      self$p_to_pf_inds <- pf_to_p
+      self$pf_to_p_log  <- p_to_pf
+      self$useC <- useC
       # Latent vars for first dim will be pinned to 0
-      p <- rnorm(latentdim*(nlevels - 1))
+      # p <- rnorm(latentdim*(nlevels - 1))
+      # Now pinning more to 0, more complex conversions
+      p <- rnorm(length(self$p_to_pf_inds))
       self$p <- p
       self$p_length <- length(p)
       # Ensure separation between levels to avoid instability
       self$p_lower <- rep(-25, self$p_length)
       # Don't give upper 1 since it will give optimization error
       self$p_upper <- rep(25, self$p_length)
-      # self$logp <- log(p, 10)
-
-      # Now set upper and lower so they have correct length
-      # self$logp_lower <- log(p_lower, 10)
-      # self$logp_upper <- log(p_upper, 10)
-      # Setting logp_lower so dimensions are right
-      # logp_lower <- log(p_lower, 10)
-      # logp_upper <- log(p_upper, 10)
-      # self$logp_lower <- if (length(logp_lower) == self$p_length) {logp_lower}
-      # else if (length(logp_lower)==1) {rep(logp_lower, self$p_length)}
-      # else {stop("Error for kernel_Periodic logp_lower")}
-      # self$logp_upper <- if (length(logp_upper) == self$p_length) {logp_upper}
-      # else if (length(logp_upper)==1) {rep(logp_upper, self$p_length)}
-      # else {stop("Error for kernel_Periodic logp_upper")}
 
       self$p_est <- p_est
       self$s2 <- s2
@@ -176,6 +184,7 @@ LatentFactorKernel <- R6::R6Class(
       self$logs2_upper <- log(s2_upper, 10)
       self$s2_est <- s2_est
 
+      self$offdiagequal <- offdiagequal
     },
     #' @description Calculate covariance between two points
     #' @param x vector.
@@ -187,55 +196,37 @@ LatentFactorKernel <- R6::R6Class(
     k = function(x, y=NULL, p=self$p, s2=self$s2, params=NULL) {
       if (!is.null(params)) {
         lenparams <- length(params)
-        # logp <- params[1:(lenpar-2)]
-        # logalpha <- params[lenpar-1]
-        # logs2 <- params[lenpar]
 
         if (self$p_est) {
           p <- params[1:self$p_length]
         } else {
           p <- self$p
         }
-        # if (self$alpha_est) {
-        #   logalpha <- params[1 + as.integer(self$p_est) * self$p_length]
-        # } else {
-        #   logalpha <- self$logalpha
-        # }
         if (self$s2_est) {
           logs2 <- params[lenparams]
         } else {
           logs2 <- self$logs2
         }
 
-
-
-
         s2 <- 10^logs2
       } else {
         if (is.null(p)) {p <- self$p}
-        # if (is.null(logalpha)) {logalpha <- self$logalpha}
         if (is.null(s2)) {s2 <- self$s2}
       }
 
-      # pf = p full has zero for first dim, then p
-      pf <- c(rep(0, self$latentdim), p)
-      # p <- 10^logp
-      # alpha <- 10^logalpha
+      pf <- self$p_to_pf(p)
       if (is.null(y)) {
         if (is.matrix(x)) {
-          # val <- outer(1:nrow(x), 1:nrow(x),
-          #              Vectorize(function(i,j){
-          #                self$kone(x[i,],x[j,],p=p, s2=s2)
-          #              }))
           if (self$useC) {
-            val <- s2 * corr_latentfactor_matrix_symC((x), pf, self$xindex, self$latentdim, 1-1e-6)
+            val <- s2 * corr_latentfactor_matrix_symC(x, pf, self$xindex,
+                                                      self$latentdim,
+                                                      self$offdiagequal)
           } else {
             val <- outer(1:nrow(x), 1:nrow(x),
                          Vectorize(function(i,j){
                            self$kone(x[i,],x[j,],pf=pf, s2=s2, isdiag=i==j)
                          }))
           }
-          # if (inherits(cgmtry,"try-error")) {browser()}
           return(val)
         } else {
           return(s2 * 1)
@@ -244,11 +235,13 @@ LatentFactorKernel <- R6::R6Class(
       if (is.matrix(x) & is.matrix(y)) {
         # C took 0.000 sec, R took 1.793 sec
         if (self$useC) { # Way faster
-          corr_latentfactor_matrixmatrixC(x=x, y=y, theta=pf, xindex=self$xindex,
-                                          latentdim = self$latentdim, offdiagequal=1-1e-6)
+          s2 * corr_latentfactor_matrixmatrixC(
+            x=x, y=y, theta=pf, xindex=self$xindex,
+            latentdim = self$latentdim, offdiagequal=self$offdiagequal)
         } else {
           outer(1:nrow(x), 1:nrow(y),
-                Vectorize(function(i,j){self$kone(x[i,],y[j,],pf=pf, s2=s2, isdiag=FALSE)}))
+                Vectorize(function(i,j){self$kone(x[i,],y[j,],
+                                                  pf=pf, s2=s2, isdiag=FALSE)}))
         }
       } else if (is.matrix(x) & !is.matrix(y)) {
         apply(x, 1, function(xx) {self$kone(xx, y, pf=pf, s2=s2)})
@@ -269,9 +262,7 @@ LatentFactorKernel <- R6::R6Class(
     #' indices are the same? Use to avoid decomposition errors, similar to
     #' adding a nugget.
     #' @references https://stackoverflow.com/questions/27086195/linear-index-upper-triangular-matrix
-    kone = function(x, y, pf, s2, isdiag=TRUE, offdiagequal=1-1e-6) {
-      # if (missing(p)) {p <- 10^logp}
-      # out <- s2 * exp(-sum(alpha*sin(p * (x-y))^2))
+    kone = function(x, y, pf, s2, isdiag=TRUE, offdiagequal=self$offdiagequal) {
       x <- x[self$xindex]
       y <- y[self$xindex]
       stopifnot(x>=1, y>=1, x<=self$nlevels, y<=self$nlevels,
@@ -306,8 +297,7 @@ LatentFactorKernel <- R6::R6Class(
     #' @param C_nonug Covariance without nugget added to diagonal
     #' @param C Covariance with nugget
     #' @param nug Value of nugget
-    dC_dparams = function(params=NULL, X, C_nonug, C, nug) {#browser(text = "Make sure all in one list")
-      # stop("not implemented, kernel index, dC_dp")
+    dC_dparams = function(params=NULL, X, C_nonug, C, nug) {
       n <- nrow(X)
 
       stopifnot(X[, self$xindex] >= 1, X[, self$xindex] <= self$nlevels)
@@ -320,11 +310,6 @@ LatentFactorKernel <- R6::R6Class(
         } else {
           p <- self$p
         }
-        # if (self$alpha_est) {
-        #   logalpha <- params[1 + as.integer(self$p_est) * self$p_length]
-        # } else {
-        #   logalpha <- self$logalpha
-        # }
         if (self$s2_est) {
           logs2 <- params[lenparams]
         } else {
@@ -332,20 +317,12 @@ LatentFactorKernel <- R6::R6Class(
         }
       } else {
         p <- self$p
-        # logalpha <- self$logalpha
         logs2 <- self$logs2
       }
 
-      # lenparams <- length(params)
-      # logp <- params[1:(lenparams - 2)]
-      # p <- 10^logp
-      # logalpha <- params[lenparams-1]
-      # alpha <- 10^logalpha
       log10 <- log(10)
-      # logs2 <- params[lenparams]
       s2 <- 10 ^ logs2
 
-      # if (is.null(params)) {params <- c(self$logp, self$logalpha, self$logs2)}
       if (missing(C_nonug)) { # Assume C missing too, must have nug
         C_nonug <- self$k(x=X, params=params)
         C <- C_nonug + diag(nug*s2, nrow(C_nonug))
@@ -353,8 +330,7 @@ LatentFactorKernel <- R6::R6Class(
 
       lenparams_D <- self$p_length*self$p_est + self$s2_est
 
-      # pf = p full has zero for first dim, then p
-      pf <- c(rep(0, self$latentdim), p)
+      pf <- self$p_to_pf(p)
 
       if (self$useC) {
         dC_dparams <- kernel_latentFactor_dC(X, pf, C_nonug, self$s2_est,
@@ -367,38 +343,48 @@ LatentFactorKernel <- R6::R6Class(
           dC_dparams[lenparams_D,,] <- C * log10
         }
 
-
         # Repeatedly calling self$ attributes is slow,
         # it's faster to just store as new variable
         latentdim <- self$latentdim
         xindex <- self$xindex
 
         if (self$p_est) {
-          # for (k in 1:length(p)) { # k is index of parameter
           stopifnot(self$nlevels>=2L)
           for (k in 2:self$nlevels) { # k is index of level
-            kinds <- (k-1)*latentdim+1:latentdim - latentdim
+            # kinds <- (k-1)*latentdim+1:latentdim - latentdim
+            kinds <- (cumsum(self$pf_to_p_log) * self$pf_to_p_log)[
+              (k-1)*latentdim+1:latentdim]
+            kinds <- kinds[kinds != 0]
+            stopifnot(length(kinds)>0, !anyDuplicated(kinds))
+            # kactiveinds <- kinds[self$pf_to_p_log[kinds]]
+            # kactiveinds <- ((kinds - 1) %% self$nlevels) + 1
+            kactiveinds <- 1:length(kinds)
+            stopifnot(length(kinds) == length(kactiveinds))
             for (i in seq(1, n-1, 1)) { # Index of X
+              xlev <- X[i, xindex]
+              latentx <- pf[(xlev-1)*latentdim+1:latentdim]
               for (j in seq(i+1, n, 1)) { # Index of Y
-                xlev <- X[i, xindex]
                 ylev <- X[j, xindex]
                 if (xlev > 1.5 && xlev == k && ylev != k) {
-                  latentx <- pf[(xlev-1)*latentdim+1:latentdim]
                   latenty <- pf[(ylev-1)*latentdim+1:latentdim]
                   p_dist2 <- sum((latentx - latenty)^2)
                   out <- s2 * exp(-p_dist2)
                   # kinds <- (xlev-1)*latentdim+1:latentdim - latentdim
-                  dC_dparams[kinds,i,j] <- -2 * out * (latentx - latenty)
+                  # dC_dparams[kinds,i,j] <- -2 * out * (latentx - latenty)
+                  # dC_dparams[kinds,j,i] <- dC_dparams[kinds,i,j]
+                  dC_dparams[kinds,i,j] <- -2 * out * (latentx[kactiveinds] -
+                                                         latenty[kactiveinds])
                   dC_dparams[kinds,j,i] <- dC_dparams[kinds,i,j]
                 } else if (ylev > 1.5 && xlev != k && ylev == k) {
-                  latentx <- pf[(xlev-1)*latentdim+1:latentdim]
+                  # latentx <- pf[(xlev-1)*latentdim+1:latentdim]
                   latenty <- pf[(ylev-1)*latentdim+1:latentdim]
                   p_dist2 <- sum((latentx - latenty)^2)
                   out <- s2 * exp(-p_dist2)
                   # kinds <- (ylev-1)*latentdim+1:latentdim - latentdim
-                  # if (inherits(try({
-                  #   dC_dparams[kinds,i,j] <- 2 * out * (latentx - latenty)}), 'try-error')) {browser()}
-                  dC_dparams[kinds,i,j] <- 2 * out * (latentx - latenty)
+                  # dC_dparams[kinds,i,j] <- 2 * out * (latentx - latenty)
+                  # dC_dparams[kinds,j,i] <- dC_dparams[kinds,i,j]
+                  dC_dparams[kinds,i,j] <- 2 * out * (latentx[kactiveinds] -
+                                                        latenty[kactiveinds])
                   dC_dparams[kinds,j,i] <- dC_dparams[kinds,i,j]
                 } else {
                   # Derivative is when when level isn't used in either
@@ -412,24 +398,6 @@ LatentFactorKernel <- R6::R6Class(
           }
         }
       }
-      # # Grad for logalpha
-      # if (self$alpha_est) {
-      #   alph_ind <- lenparams_D - as.integer(self$s2_est)
-      #   for (i in seq(1, n-1, 1)) {
-      #     for (j in seq(i+1, n, 1)) {
-      #       r2 <- -sum(sin(p * (X[i,]-X[j,]))^2)
-      #       dC_dparams[alph_ind, i,j] <- C_nonug[i,j] * r2 * alpha * log10
-      #       dC_dparams[alph_ind, j,i] <- dC_dparams[alph_ind, i,j]
-      #     }
-      #   }
-      #   for (i in seq(1, n, 1)) {
-      #     dC_dparams[alph_ind, i,i] <- 0
-      #   }
-      # }
-      # cout <- kernel_latentFactor_dC(X, pf, C_nonug, self$s2_est,
-      #                                self$p_est, lenparams_D, s2*nug,
-      #                                self$latentdim, self$xindex-1,
-      #                                self$nlevels, s2)
       return(dC_dparams)
     },
     #' @description Calculate covariance matrix and its derivative
@@ -447,47 +415,32 @@ LatentFactorKernel <- R6::R6Class(
     #' @description Derivative of covariance with respect to X
     #' @param XX matrix of points
     #' @param X matrix of points to take derivative with respect to
-    #' @param logp log of p
-    #' @param logalpha log of alpha
-    #' @param s2 Variance parameter
-    dC_dx = function(XX, X, logp=self$logp, logalpha=self$logalpha, s2=self$s2) {
-      stop("not implemented, latent factor kernel, dC_dx")
-      # if (missing(theta)) {theta <- 10^beta}
-      p <- 10 ^ logp
-      alpha <- 10 ^ logalpha
+    #' @param ... Additional args, not used
+    dC_dx = function(XX, X, ...) {
       if (!is.matrix(XX)) {stop()}
       d <- ncol(XX)
       if (ncol(X) != d) {stop()}
       n <- nrow(X)
       nn <- nrow(XX)
-
-      # pf = p full has zero for first dim, then p
-      pf <- c(rep(0, self$latentdim), p)
-
-      dC_dx <- array(NA, dim=c(nn, d, n))
-      for (i in 1:nn) {
-        for (j in 1:d) {
-          for (k in 1:n) {
-            # r <- sqrt(sum(theta * (XX[i,] - X[k,]) ^ 2))
-            CC <- s2 * exp(-sum(alpha * sin(p * (XX[i, ]-X[k, ]))^2))
-            dC_dx[i, j, k] <- CC * (-alpha) * sin(2*p[j]*(XX[i, j]-X[k, j])) * p[j] #* (XX[i, j] - X[k, j])
-          }
-        }
-      }
+      dC_dx <- array(0, dim=c(nn, d, n))
+      dC_dx[, self$xindex, ] <- NA
       dC_dx
     },
     #' @description Starting point for parameters for optimization
     #' @param jitter Should there be a jitter?
     #' @param y Output
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_start = function(jitter=F, y, p_est=self$p_est,
                                  s2_est=self$s2_est) {
-      if (p_est) {vec <- c(self$p)} else {vec <- c()}
-      if (s2_est) {vec <- c(vec, self$logs2)} else {}
-      if (jitter && p_est) {
-        vec[1:length(self$p)] = vec[1:length(self$p)] + rnorm(length(self$p), 0, 1)
+      if (p_est) {
+        vec <- pmin(pmax(self$p + jitter*rnorm(length(self$p), 0, 1),
+                         self$p_lower), self$p_upper)
+      } else {
+        vec <- c()
+      }
+      if (s2_est) {
+        vec <- c(vec, self$logs2 + jitter*rnorm(1))
       }
       vec
     },
@@ -495,20 +448,22 @@ LatentFactorKernel <- R6::R6Class(
     #' @param jitter Should there be a jitter?
     #' @param y Output
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_start0 = function(jitter=F, y, p_est=self$p_est,
                                   s2_est=self$s2_est) {
-      if (p_est) {vec <- rnorm(self$p_length)} else {vec <- c()}
-      if (s2_est) {vec <- c(vec, 0)} else {}
-      if (jitter && p_est) {
-        vec[1:length(self$p)] = vec[1:length(self$logp)] + rnorm(length(self$p), 0, 1)
+      if (p_est) {
+        vec <- pmin(pmax(jitter*rnorm(length(self$p), 0, 1),
+                         self$p_lower), self$p_upper)
+      } else {
+        vec <- c()
+      }
+      if (s2_est) {
+        vec <- c(vec, self$logs2 + jitter*rnorm(1))
       }
       vec
     },
     #' @description Lower bounds of parameters for optimization
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_lower = function(p_est=self$p_est,
                                  s2_est=self$s2_est) {
@@ -518,7 +473,6 @@ LatentFactorKernel <- R6::R6Class(
     },
     #' @description Upper bounds of parameters for optimization
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     param_optim_upper = function(p_est=self$p_est,
                                  s2_est=self$s2_est) {
@@ -529,7 +483,6 @@ LatentFactorKernel <- R6::R6Class(
     #' @description Set parameters from optimization output
     #' @param optim_out Output from optimization
     #' @param p_est Is p being estimated?
-    #' @param alpha_est Is alpha being estimated?
     #' @param s2_est Is s2 being estimated?
     set_params_from_optim = function(optim_out, p_est=self$p_est,
                                      s2_est=self$s2_est) {
@@ -542,6 +495,14 @@ LatentFactorKernel <- R6::R6Class(
         self$s2 <- 10 ^ self$logs2
       }
     },
+    #' @description Convert p (short parameter vector) to pf (long parameter
+    #' vector with zeros).
+    #' @param p Parameter vector
+    p_to_pf = function(p) {
+      pf <- rep(0, length(self$pf_to_p_log))
+      pf[self$pf_to_p_log] <- p
+      pf
+    },
     #' @description Get s2 from params vector
     #' @param params parameter vector
     #' @param s2_est Is s2 being estimated?
@@ -552,41 +513,22 @@ LatentFactorKernel <- R6::R6Class(
         self$s2
       }
     },
-    #' @param ... Not used.
-    plot = function(...) {
-      x1 <- 1:self$nlevels
-      X1 <- X2 <- matrix(data=0, ncol=self$D, nrow=self$nlevels)
-      X1[, self$xindex] <- x1
-      X2[, self$xindex] <- x1
-      k <- self$k(X1, X2)
-
-      df <- NULL
-      for (i in 1:self$nlevels) {
-        for (j in 1:self$nlevels) {
-          df <- rbind(df,
-                      data.frame(x1=i, x2=j, k=k[i,j]))
-        }
-      }
-      ggplot2::ggplot(data=df, ggplot2::aes(x1, x2, fill=k)) +
-        ggplot2::geom_tile() +
-        ggplot2::scale_fill_gradient(low='white', high='red', limits=c(0, NA)) +
-        ggplot2::scale_y_reverse()
-    },
     #' @description Plot the points in the latent space
     plotLatent = function() {
-      # pf = p full has zero for first dim, then p
-      pf <- c(rep(0, self$latentdim), self$p)
-
+      pf <- self$p_to_pf(self$p)
       pmat <- matrix(pf, ncol=self$latentdim, byrow=TRUE)
       pdf <- as.data.frame(pmat)
-      pdf$name <- paste0("D=",1:nrow(pdf))
+      pdf$name <- paste0("x=",1:nrow(pdf))
       if (self$latentdim == 1) {
         ggplot2::ggplot(pdf, ggplot2::aes(V1, 0, label=name)) +
           ggplot2::geom_point() +
-          ggrepel::geom_label_repel()
+          ggplot2::scale_y_continuous(breaks=NULL) +
+          ggrepel::geom_label_repel() +
+          ggplot2::ylab(NULL)
       } else if (self$latentdim == 2) {
         ggplot2::ggplot(pdf, ggplot2::aes(V1, V2, label=name)) +
-          ggplot2::geom_point() + ggrepel::geom_label_repel()
+          ggplot2::geom_point() +
+          ggrepel::geom_label_repel()
       } else {
         stop("Can't plotLatent for latentdim > 2")
       }
